@@ -27,6 +27,7 @@ from __future__ import print_function
 import sys
 import math
 import os
+import textwrap
 
 import inspect
 from datetime import datetime
@@ -50,19 +51,30 @@ from myhdl.conversion._misc import (_error, _kind, _context,
 from myhdl.conversion._analyze import (_analyzeSigs, _analyzeGens, _analyzeTopFunc,
                                        _Ram, _Rom)
 from myhdl._Signal import _Signal
+from myhdl._ShadowSignal import _TristateSignal, _TristateDriver
 
+from myhdl._block import _Block
+from myhdl._getHierarchy import _getHierarchy
 
 _converting = 0
 _profileFunc = None
 
+
 def _checkArgs(arglist):
     for arg in arglist:
         if not isinstance(arg, (GeneratorType, _Instantiator, _UserVerilogCode)):
             raise ToVerilogError(_error.ArgType, arg)
 
+
 def _flatten(*args):
     arglist = []
     for arg in args:
+        if isinstance(arg, _Block):
+            if arg.verilog_code is not None:
+                arglist.append(arg.verilog_code)
+                continue
+            else:
+                arg = arg.subs
         if id(arg) in _userCodeMap['verilog']:
             arglist.append(_userCodeMap['verilog'][id(arg)])
         elif isinstance(arg, (list, tuple, set)):
@@ -95,7 +107,8 @@ class _ToVerilogConvertor(object):
                  "no_myhdl_header",
                  "no_testbench",
                  "portmap",
-                 "trace"
+                 "trace",
+                 "initial_values"
                  )
 
     def __init__(self):
@@ -109,29 +122,42 @@ class _ToVerilogConvertor(object):
         self.no_myhdl_header = False
         self.no_testbench = False
         self.trace = False
+        self.initial_values = False
 
     def __call__(self, func, *args, **kwargs):
         global _converting
         if _converting:
-            return func(*args, **kwargs) # skip
+            return func(*args, **kwargs)  # skip
         else:
             # clean start
             sys.setprofile(None)
         from myhdl import _traceSignals
         if _traceSignals._tracing:
             raise ToVerilogError("Cannot use toVerilog while tracing signals")
-        if not callable(func):
-            raise ToVerilogError(_error.FirstArgType, "got %s" % type(func))
+        if not isinstance(func, _Block):
+            if not callable(func):
+                raise ToVerilogError(_error.FirstArgType, "got %s" % type(func))
 
         _converting = 1
         if self.name is None:
             name = func.__name__
+            if isinstance(func, _Block):
+                name = func.func.__name__
         else:
             name = str(self.name)
-        try:
-            h = _HierExtr(name, func, *args, **kwargs)
-        finally:
-            _converting = 0
+
+        if isinstance(func, _Block):
+            try:
+                h = _getHierarchy(name, func)
+            finally:
+                _converting = 0
+        else:
+            warnings.warn(
+                "\n    toVerilog(): Deprecated usage: See http://dev.myhdl.org/meps/mep-114.html", stacklevel=2)
+            try:
+                h = _HierExtr(name, func, *args, **kwargs)
+            finally:
+                _converting = 0
 
         if self.directory is None:
             directory = ''
@@ -151,9 +177,17 @@ class _ToVerilogConvertor(object):
         genlist = _analyzeGens(arglist, h.absnames)
         siglist, memlist = _analyzeSigs(h.hierarchy)
         _annotateTypes(genlist)
-        top_inst = h.hierarchy[0]
-        intf = _analyzeTopFunc(top_inst, func, *args, **kwargs)
+
+        # infer interface
+        if isinstance(func, _Block):
+            # infer interface after signals have been analyzed
+            func._inferInterface()
+            intf = func
+        else:
+            intf = _analyzeTopFunc(func, *args, **kwargs)
+
         intf.name = name
+
         doc = _makeDoc(inspect.getdoc(func))
 
         self._convert_filter(h, intf, siglist, memlist, genlist)
@@ -176,22 +210,25 @@ class _ToVerilogConvertor(object):
         # build portmap for cosimulation
         portmap = {}
         for n, s in intf.argdict.items():
-            if hasattr(s, 'driver'): portmap[n] = s.driver()
-            else: portmap[n] = s
+            if hasattr(s, 'driver'):
+                portmap[n] = s.driver()
+            else:
+                portmap[n] = s
         self.portmap = portmap
 
         ### clean-up properly ###
-        self._cleanup(siglist)
+        self._cleanup(siglist, memlist)
 
         return h.top
 
-    def _cleanup(self, siglist):
-        # clean up signal names
+    def _cleanup(self, siglist, memlist):
+        # clean up signals
         for sig in siglist:
             sig._clear()
-#             sig._name = None
-#             sig._driven = False
-#             sig._read = False
+        for mem in memlist:
+            mem.name = None
+            for s in mem.mem:
+                s._clear()
 
         # clean up attributes
         self.name = None
@@ -203,7 +240,6 @@ class _ToVerilogConvertor(object):
         self.no_testbench = False
         self.trace = False
 
-
     def _convert_filter(self, h, intf, siglist, memlist, genlist):
         # intended to be a entry point for other uses:
         #  code checking, optimizations, etc
@@ -218,6 +254,7 @@ myhdl_header = """\
 // Date: $date
 """
 
+
 def _writeFileHeader(f, fn, ts):
     vars = dict(filename=fn,
                 version=myhdl.__version__,
@@ -254,10 +291,14 @@ def _writeModuleHeader(f, intf, doc):
         p = _getSignString(s)
         if s._driven:
             if s._read:
-                warnings.warn("%s: %s" % (_error.OutputPortRead, portname),
-                              category=ToVerilogWarning
-                              )
-            print("output %s%s%s;" % (p, r, portname), file=f)
+                if not isinstance(s, _TristateSignal):
+                    warnings.warn("%s: %s" % (_error.OutputPortRead, portname),
+                                  category=ToVerilogWarning
+                                  )
+            if isinstance(s, _TristateSignal):
+                print("inout %s%s%s;" % (p, r, portname), file=f)
+            else:
+                print("output %s%s%s;" % (p, r, portname), file=f)
             if s._driven == 'reg':
                 print("reg %s%s%s;" % (p, r, portname), file=f)
             else:
@@ -281,7 +322,7 @@ def _writeSigDecls(f, intf, siglist, memlist):
         r = _getRangeString(s)
         p = _getSignString(s)
         if s._driven:
-            if not s._read:
+            if not s._read and not isinstance(s, _TristateDriver):
                 warnings.warn("%s: %s" % (_error.UnreadSignal, s._name),
                               category=ToVerilogWarning
                               )
@@ -289,8 +330,17 @@ def _writeSigDecls(f, intf, siglist, memlist):
             if s._driven == 'reg':
                 k = 'reg'
             # the following line implements initial value assignments
-            # print >> f, "%s %s%s = %s;" % (k, r, s._name, int(s._val))
-            print("%s %s%s%s;" % (k, p, r, s._name), file=f)
+            # don't initial value "wire", inital assignment to a wire
+            # equates to a continuous assignment [reference]
+            if not toVerilog.initial_values or k == 'wire':
+                print("%s %s%s%s;" % (k, p, r, s._name), file=f)
+            else:
+                if isinstance(s._init, myhdl._enum.EnumItemType):
+                    print("%s %s%s%s = %s;" %
+                          (k, p, r, s._name, s._init._toVerilog()), file=f)
+                else:
+                    print("%s %s%s%s = %s;" %
+                          (k, p, r, s._name, _intRepr(s._init)), file=f)
         elif s._read:
             # the original exception
             # raise ToVerilogError(_error.UndrivenSignal, s._name)
@@ -300,7 +350,7 @@ def _writeSigDecls(f, intf, siglist, memlist):
                           )
             constwires.append(s)
             print("wire %s%s;" % (r, s._name), file=f)
-    print(file=f)
+    # print(file=f)
     for m in memlist:
         if not m._used:
             continue
@@ -315,17 +365,52 @@ def _writeSigDecls(f, intf, siglist, memlist):
         r = _getRangeString(m.elObj)
         p = _getSignString(m.elObj)
         k = 'wire'
+        initial_assignments = None
         if m._driven:
             k = m._driven
-        print("%s %s%s%s [0:%s-1];" % (k, p, r, m.name, m.depth), file=f)
+
+            if toVerilog.initial_values and not k == 'wire':
+                if all([each._init == m.mem[0]._init for each in m.mem]):
+
+                    initialize_block_name = ('INITIALIZE_' + m.name).upper()
+                    _initial_assignments = (
+                        '''
+                        initial begin: %s
+                            integer i;
+                            for(i=0; i<%d; i=i+1) begin
+                                %s[i] = %s;
+                            end
+                        end
+                        ''' % (initialize_block_name, len(m.mem), m.name,
+                               _intRepr(m.mem[0]._init)))
+
+                    initial_assignments = (
+                        textwrap.dedent(_initial_assignments))
+
+                else:
+                    val_assignments = '\n'.join(
+                        ['    %s[%d] <= %s;' %
+                         (m.name, n, _intRepr(each._init))
+                         for n, each in enumerate(m.mem)])
+                    initial_assignments = (
+                        'initial begin\n' + val_assignments + '\nend')
+
+        print("%s %s%s%s [0:%s-1];" % (k, p, r, m.name, m.depth),
+              file=f)
+
+        if initial_assignments is not None:
+            print(initial_assignments, file=f)
+
     print(file=f)
     for s in constwires:
         if s._type in (bool, intbv):
             c = int(s.val)
         else:
             raise ToVerilogError("Unexpected type for constant signal", s._name)
-        print("assign %s = %s;" % (s._name, c), file=f)
-    print(file=f)
+        c_len = s._nrbits
+        c_str = "%s" % c
+        print("assign %s = %s'd%s;" % (s._name, c_len, c_str), file=f)
+    # print(file=f)
     # shadow signal assignments
     for s in siglist:
         if hasattr(s, 'toVerilog') and s._driven:
@@ -380,16 +465,35 @@ def _getRangeString(s):
         return ''
     elif s._nrbits is not None:
         nrbits = s._nrbits
-        return "[%s:0] " % (nrbits-1)
+        return "[%s:0] " % (nrbits - 1)
     else:
         raise AssertionError
 
+
 def _getSignString(s):
     if s._min is not None and s._min < 0:
         return "signed "
     else:
         return ''
 
+def _intRepr(n, radix=''):
+    # write size for large integers (beyond 32 bits signed)
+    # with some safety margin
+    # XXX signed indication 's' ???
+    p = abs(n)
+    size = ''
+    num = str(p).rstrip('L')
+    if radix == "hex" or p >= 2**30:
+        radix = "'h"
+        num = hex(p)[2:].rstrip('L')
+    if p >= 2**30:
+        size = int(math.ceil(math.log(p+1,2))) + 1  # sign bit!
+#            if not radix:
+#                radix = "'d"
+    r = "%s%s%s" % (size, radix, num)
+    if n < 0: # add brackets and sign on negative numbers
+        r = "(-%s)" % r
+    return r
 
 def _convertGens(genlist, vfile):
     blockBuf = StringIO()
@@ -408,39 +512,41 @@ def _convertGens(genlist, vfile):
             Visitor = _ConvertAlwaysDecoVisitor
         elif tree.kind == _kind.ALWAYS_SEQ:
             Visitor = _ConvertAlwaysSeqVisitor
-        else: # ALWAYS_COMB
+        else:  # ALWAYS_COMB
             Visitor = _ConvertAlwaysCombVisitor
         v = Visitor(tree, blockBuf, funcBuf)
         v.visit(tree)
-    vfile.write(funcBuf.getvalue()); funcBuf.close()
-    vfile.write(blockBuf.getvalue()); blockBuf.close()
+    vfile.write(funcBuf.getvalue())
+    funcBuf.close()
+    vfile.write(blockBuf.getvalue())
+    blockBuf.close()
 
 
 opmap = {
-    ast.Add      : '+',
-    ast.Sub      : '-',
-    ast.Mult     : '*',
-    ast.Div      : '/',
-    ast.Mod      : '%',
-    ast.Pow      : '**',
-    ast.LShift   : '<<',
-    ast.RShift   : '>>>',
-    ast.BitOr    : '|',
-    ast.BitAnd   : '&',
-    ast.BitXor   : '^',
-    ast.FloorDiv : '/',
-    ast.Invert   : '~',
-    ast.Not      : '!',
-    ast.UAdd     : '+',
-    ast.USub     : '-',
-    ast.Eq       : '==',
-    ast.Gt       : '>',
-    ast.GtE      : '>=',
-    ast.Lt       : '<',
-    ast.LtE      : '<=',
-    ast.NotEq    : '!=',
-    ast.And      : '&&',
-    ast.Or       : '||',
+    ast.Add: '+',
+    ast.Sub: '-',
+    ast.Mult: '*',
+    ast.Div: '/',
+    ast.Mod: '%',
+    ast.Pow: '**',
+    ast.LShift: '<<',
+    ast.RShift: '>>>',
+    ast.BitOr: '|',
+    ast.BitAnd: '&',
+    ast.BitXor: '^',
+    ast.FloorDiv: '/',
+    ast.Invert: '~',
+    ast.Not: '!',
+    ast.UAdd: '+',
+    ast.USub: '-',
+    ast.Eq: '==',
+    ast.Gt: '>',
+    ast.GtE: '>=',
+    ast.Lt: '<',
+    ast.LtE: '<=',
+    ast.NotEq: '!=',
+    ast.And: '&&',
+    ast.Or: '||',
 }
 
 nameconstant_map = {
@@ -450,7 +556,6 @@ nameconstant_map = {
 }
 
 
-
 class _ConvertVisitor(ast.NodeVisitor, _ConversionMixin):
 
     def __init__(self, tree, buf):
@@ -466,7 +571,7 @@ class _ConvertVisitor(ast.NodeVisitor, _ConversionMixin):
     def raiseError(self, node, kind, msg=""):
         lineno = self.getLineNo(node)
         info = "in file %s, line %s:\n    " % \
-              (self.tree.sourcefile, self.tree.lineoffset+lineno)
+            (self.tree.sourcefile, self.tree.lineoffset + lineno)
         raise ToVerilogError(kind, msg, info)
 
     def write(self, arg):
@@ -489,26 +594,11 @@ class _ConvertVisitor(ast.NodeVisitor, _ConversionMixin):
         self.ind = self.ind[:-4]
 
     def IntRepr(self, n, radix=''):
-        # write size for large integers (beyond 32 bits signed)
-        # with some safety margin
-        # XXX signed indication 's' ???
-        p = abs(n)
-        size = ''
-        num = str(p).rstrip('L')
-        if radix == "hex" or p >= 2**30:
-            radix = "'h"
-            num = hex(p)[2:].rstrip('L')
-        if p >= 2**30:
-            size = int(math.ceil(math.log(p+1,2))) + 1  # sign bit!
-#            if not radix:
-#                radix = "'d"
-        r = "%s%s%s" % (size, radix, num)
-        if n < 0: # add brackets and sign on negative numbers
-            r = "(-%s)" % r
-        return r
+        return _intRepr(n, radix)
 
     def writeDeclaration(self, obj, name, dir):
-        if dir: dir = dir + ' '
+        if dir:
+            dir = dir + ' '
         if type(obj) is bool:
             self.write("%s%s" % (dir, name))
         elif isinstance(obj, int):
@@ -568,7 +658,7 @@ class _ConvertVisitor(ast.NodeVisitor, _ConversionMixin):
             self.write(", ")
             self.visit(node.right)
         else:
-            if isinstance(node.op,  ast.RShift):
+            if isinstance(node.op, ast.RShift):
                 # Additional cast to signed of the full expression
                 # this is apparently required by cver - not sure if it
                 # is actually required by standard Verilog.
@@ -676,7 +766,7 @@ class _ConvertVisitor(ast.NodeVisitor, _ConversionMixin):
             self.indent()
             for i, n in enumerate(rom):
                 self.writeline()
-                if i == len(rom)-1:
+                if i == len(rom) - 1:
                     self.write("default: ")
                 else:
                     self.write("%s: " % i)
@@ -687,7 +777,7 @@ class _ConvertVisitor(ast.NodeVisitor, _ConversionMixin):
                 else:
                     self.write(' = ')
                 s = self.IntRepr(n)
-                self.write("%s;" %s)
+                self.write("%s;" % s)
             self.dedent()
             self.writeline()
             self.write("endcase")
@@ -744,11 +834,7 @@ class _ConvertVisitor(ast.NodeVisitor, _ConversionMixin):
             return
         elif f is ord:
             opening, closing = '', ''
-            if isinstance(node.args[0], ast.Str):
-                if len(node.args[0].s) > 1:
-                    self.raiseError(node, _error.UnsupportedType, "Strings with length > 1")
-                else:
-                    node.args[0].s = str(ord(node.args[0].s))
+            node.args[0].s = str(ord(node.args[0].s))
         elif f in integer_types:
             opening, closing = '', ''
             # convert number argument to integer
@@ -757,7 +843,7 @@ class _ConvertVisitor(ast.NodeVisitor, _ConversionMixin):
         elif f in (intbv, modbv):
             self.visit(node.args[0])
             return
-        elif f == intbv.signed: # note equality comparison
+        elif f == intbv.signed:  # note equality comparison
             # comes from a getattr
             opening, closing = '', ''
             if not fn.value.signed:
@@ -863,10 +949,10 @@ class _ConvertVisitor(ast.NodeVisitor, _ConversionMixin):
                 start, stop, step = args[0], args[1], None
             else:
                 start, stop, step = args
-        else: # downrange
+        else:  # downrange
             cmp = '>='
             op = '-'
-            oneoff ='-1'
+            oneoff = '-1'
             if len(args) == 1:
                 start, stop, step = args[0], None, None
             elif len(args) == 2:
@@ -991,7 +1077,7 @@ class _ConvertVisitor(ast.NodeVisitor, _ConversionMixin):
             self.visit(stmt)
 
     def visit_ListComp(self, node):
-        pass # do nothing
+        pass  # do nothing
 
     def visit_NameConstant(self, node):
         self.write(nameconstant_map[node.obj])
@@ -1023,7 +1109,7 @@ class _ConvertVisitor(ast.NodeVisitor, _ConversionMixin):
         elif n in self.tree.symdict:
             obj = self.tree.symdict[n]
             if isinstance(obj, bool):
-                s = "%s" % int(obj)
+                s = "1'b%s" % int(obj)
             elif isinstance(obj, integer_types):
                 s = self.IntRepr(obj)
             elif isinstance(obj, _Signal):
@@ -1059,8 +1145,11 @@ class _ConvertVisitor(ast.NodeVisitor, _ConversionMixin):
                 a = node.args[argnr]
                 argnr += 1
                 obj = a.obj
-                fs = "%0d"
-                self.context =_context.PRINT
+                if s.conv is int or isinstance(obj, int):
+                    fs = "%0d"
+                else:
+                    fs = "%h"
+                self.context = _context.PRINT
                 if isinstance(obj, str):
                     self.write('$write(')
                     self.visit(a)
@@ -1090,6 +1179,7 @@ class _ConvertVisitor(ast.NodeVisitor, _ConversionMixin):
                     self.writeline()
                     self.write("endcase")
                 else:
+                    print (type(obj), type(a))
                     self.write('$write("%s", ' % fs)
                     self.visit(a)
                     self.write(');')
@@ -1142,8 +1232,8 @@ class _ConvertVisitor(ast.NodeVisitor, _ConversionMixin):
 
     def accessIndex(self, node):
         addSignBit = isinstance(node.ctx, ast.Load) and \
-                     (not node.signed) and \
-                     (self.context == _context.SIGNED)
+            (not node.signed) and \
+            (self.context == _context.SIGNED)
         if addSignBit:
             self.write("$signed({1'b0, ")
         self.context = None
@@ -1252,7 +1342,6 @@ class _ConvertInitialVisitor(_ConvertVisitor):
         self.writeline(2)
 
 
-
 class _ConvertAlwaysCombVisitor(_ConvertVisitor):
 
     def __init__(self, tree, blockBuf, funcBuf):
@@ -1272,12 +1361,12 @@ class _ConvertAlwaysCombVisitor(_ConvertVisitor):
         self.writeline(2)
 
 
-
 class _ConvertSimpleAlwaysCombVisitor(_ConvertVisitor):
 
     def __init__(self, tree, blockBuf, funcBuf):
         _ConvertVisitor.__init__(self, tree, blockBuf)
         self.funcBuf = funcBuf
+
     def visit_Attribute(self, node):
         if isinstance(node.ctx, ast.Store):
             self.write("assign ")
@@ -1317,6 +1406,7 @@ def _convertInitVal(reg, init):
     if tipe is bool:
         v = '1' if init else '0'
     elif tipe is intbv:
+        init = int(init) # int representation
         v = "%s" % init if init is not None else "'bz"
     else:
         assert isinstance(init, EnumItemType)
@@ -1408,8 +1498,6 @@ class _ConvertFunctionVisitor(_ConvertVisitor):
         self.write("disable %s;" % self.returnLabel)
 
 
-
-
 class _ConvertTaskVisitor(_ConvertVisitor):
 
     def __init__(self, tree, funcBuf):
@@ -1451,6 +1539,7 @@ def _maybeNegative(obj):
         return True
     return False
 
+
 class _AnnotateTypesVisitor(ast.NodeVisitor, _ConversionMixin):
 
     def __init__(self, tree):
@@ -1499,7 +1588,7 @@ class _AnnotateTypesVisitor(ast.NodeVisitor, _ConversionMixin):
         self.generic_visit(node)
         f = self.getObj(node.func)
         node.signed = False
-        ### suprize: identity comparison on unbound methods doesn't work in python 2.5??
+        # suprize: identity comparison on unbound methods doesn't work in python 2.5??
         if f == intbv.signed:
             node.signed = True
         elif hasattr(node, 'tree'):
@@ -1509,7 +1598,7 @@ class _AnnotateTypesVisitor(ast.NodeVisitor, _ConversionMixin):
 
     def visit_Compare(self, node):
         node.signed = False
-        #for n in ast.iter_child_nodes(node):
+        # for n in ast.iter_child_nodes(node):
         for n in [node.left] + node.comparators:
             self.visit(n)
             if n.signed:
@@ -1520,7 +1609,6 @@ class _AnnotateTypesVisitor(ast.NodeVisitor, _ConversionMixin):
             return
         self.generic_visit(node)
 
-
     def visit_Num(self, node):
         node.signed = False
 
@@ -1564,6 +1652,3 @@ def _annotateTypes(genlist):
             continue
         v = _AnnotateTypesVisitor(tree)
         v.visit(tree)
-
-
-
