@@ -1,7 +1,7 @@
 #  This file is part of the myhdl library, a Python package for using
 #  Python as a Hardware Description Language.
 #
-#  Copyright (C) 2003-2013 Jan Decaluwe
+#  Copyright (C) 2003-2015 Jan Decaluwe
 #
 #  The myhdl library is free software; you can redistribute it and/or
 #  modify it under the terms of the GNU Lesser General Public License as
@@ -20,16 +20,14 @@
 """ Module with the intbv class """
 from __future__ import absolute_import, division
 
-import operator
 
 from myhdl._compat import long, integer_types, string_types, builtins
 from myhdl._bin import bin
 
 
-
 class intbv(object):
     #__slots__ = ('_val', '_min', '_max', '_nrbits', '_handleBounds')
-    
+
     def __init__(self, val=0, min=None, max=None, _nrbits=0):
         if _nrbits:
             self._min = 0
@@ -39,12 +37,12 @@ class intbv(object):
             self._max = max
             if max is not None and min is not None:
                 if min >= 0:
-                    _nrbits = len(bin(max-1))
+                    _nrbits = len(bin(max - 1))
                 elif max <= 1:
                     _nrbits = len(bin(min))
                 else:
                     # make sure there is a leading zero bit in positive numbers
-                    _nrbits = builtins.max(len(bin(max-1))+1, len(bin(min)))
+                    _nrbits = builtins.max(len(bin(max - 1)) + 1, len(bin(min)))
         if isinstance(val, integer_types):
             self._val = val
         elif isinstance(val, string_types):
@@ -60,7 +58,7 @@ class intbv(object):
             raise TypeError("intbv constructor arg should be int or string")
         self._nrbits = _nrbits
         self._handleBounds()
-        
+
     # support for the 'min' and 'max' attribute
     @property
     def max(self):
@@ -79,20 +77,19 @@ class intbv(object):
             if self._val < self._min:
                 raise ValueError("intbv value %s < minimum %s" %
                                  (self._val, self._min))
-                
+
     def _hasFullRange(self):
         min, max = self._min, self._max
         if max <= 0:
             return False
         if min not in (0, -max):
             return False
-        return max & max-1 == 0
-
+        return max & max - 1 == 0
 
     # hash
     def __hash__(self):
         raise TypeError("intbv objects are unhashable")
-        
+
     # copy methods
     def __copy__(self):
         c = type(self)(self._val)
@@ -112,7 +109,7 @@ class intbv(object):
     def __iter__(self):
         if not self._nrbits:
             raise TypeError("Cannot iterate over unsized intbv")
-        return iter([self[i] for i in range(self._nrbits-1, -1, -1)])
+        return iter([self[i] for i in range(self._nrbits - 1, -1, -1)])
 
     # logical testing
     def __bool__(self):
@@ -129,52 +126,50 @@ class intbv(object):
     def __getitem__(self, key):
         if isinstance(key, slice):
             i, j = key.start, key.stop
-            if j is None: # default
+            if j is None:  # default
                 j = 0
             j = int(j)
             if j < 0:
-                raise ValueError("intbv[i:j] requires j >= 0\n" \
-                      "            j == %s" % j)
-            if i is None: # default
+                raise ValueError("intbv[i:j] requires j >= 0\n"
+                                 "            j == %s" % j)
+            if i is None:  # default
                 return intbv(self._val >> j)
             i = int(i)
             if i <= j:
-                raise ValueError("intbv[i:j] requires i > j\n" \
-                      "            i, j == %s, %s" % (i, j))
-            res = intbv((self._val & (long(1) << i)-1) >> j, _nrbits=i-j)
+                raise ValueError("intbv[i:j] requires i > j\n"
+                                 "            i, j == %s, %s" % (i, j))
+            res = intbv((self._val & (long(1) << i) - 1) >> j, _nrbits=i - j)
             return res
         else:
             i = int(key)
             res = bool((self._val >> i) & 0x1)
             return res
 
-
-       
     def __setitem__(self, key, val):
         # convert val to int to avoid confusion with intbv or Signals
         val = int(val)
         if isinstance(key, slice):
             i, j = key.start, key.stop
-            if j is None: # default
+            if j is None:  # default
                 j = 0
             j = int(j)
             if j < 0:
-                raise ValueError("intbv[i:j] = v requires j >= 0\n" \
-                      "            j == %s" % j)
-            if i is None: # default
+                raise ValueError("intbv[i:j] = v requires j >= 0\n"
+                                 "            j == %s" % j)
+            if i is None:  # default
                 q = self._val % (long(1) << j)
                 self._val = val * (long(1) << j) + q
                 self._handleBounds()
                 return
             i = int(i)
             if i <= j:
-                raise ValueError("intbv[i:j] = v requires i > j\n" \
-                      "            i, j, v == %s, %s, %s" % (i, j, val))
-            lim = (long(1) << (i-j))
+                raise ValueError("intbv[i:j] = v requires i > j\n"
+                                 "            i, j, v == %s, %s, %s" % (i, j, val))
+            lim = (long(1) << (i - j))
             if val >= lim or val < -lim:
-                raise ValueError("intbv[i:j] = v abs(v) too large\n" \
-                      "            i, j, v == %s, %s, %s" % (i, j, val))
-            mask = (lim-1) << j
+                raise ValueError("intbv[i:j] = v abs(v) too large\n"
+                                 "            i, j, v == %s, %s, %s" % (i, j, val))
+            mask = (lim - 1) << j
             self._val &= ~mask
             self._val |= (val << j)
             self._handleBounds()
@@ -185,28 +180,28 @@ class intbv(object):
             elif val == 0:
                 self._val &= ~(long(1) << i)
             else:
-                raise ValueError("intbv[i] = v requires v in (0, 1)\n" \
-                      "            i == %s " % i)
-               
+                raise ValueError("intbv[i] = v requires v in (0, 1)\n"
+                                 "            i == %s " % i)
+
             self._handleBounds()
 
-
-        
     # integer-like methods
-    
+
     def __add__(self, other):
         if isinstance(other, intbv):
             return self._val + other._val
         else:
             return self._val + other
+
     def __radd__(self, other):
         return other + self._val
-    
+
     def __sub__(self, other):
         if isinstance(other, intbv):
             return self._val - other._val
         else:
             return self._val - other
+
     def __rsub__(self, other):
         return other - self._val
 
@@ -215,6 +210,7 @@ class intbv(object):
             return self._val * other._val
         else:
             return self._val * other
+
     def __rmul__(self, other):
         return other * self._val
 
@@ -223,32 +219,36 @@ class intbv(object):
             return self._val / other._val
         else:
             return self._val / other
+
     def __rtruediv__(self, other):
         return other / self._val
-    
+
     def __floordiv__(self, other):
         if isinstance(other, intbv):
             return self._val // other._val
         else:
             return self._val // other
+
     def __rfloordiv__(self, other):
-        return other //  self._val
-    
+        return other // self._val
+
     def __mod__(self, other):
         if isinstance(other, intbv):
             return self._val % other._val
         else:
             return self._val % other
+
     def __rmod__(self, other):
         return other % self._val
 
     # divmod
-    
+
     def __pow__(self, other):
         if isinstance(other, intbv):
             return self._val ** other._val
         else:
             return self._val ** other
+
     def __rpow__(self, other):
         return other ** self._val
 
@@ -257,40 +257,45 @@ class intbv(object):
             return intbv(long(self._val) << other._val)
         else:
             return intbv(long(self._val) << other)
+
     def __rlshift__(self, other):
         return other << self._val
-            
+
     def __rshift__(self, other):
         if isinstance(other, intbv):
-            return type(self)(self._val >> other._val)
+            return intbv(self._val >> other._val)
         else:
-            return type(self)(self._val >> other)
+            return intbv(self._val >> other)
+
     def __rrshift__(self, other):
         return other >> self._val
-           
+
     def __and__(self, other):
         if isinstance(other, intbv):
-            return type(self)(self._val & other._val)
+            return intbv(self._val & other._val)
         else:
-            return type(self)(self._val & other)
+            return intbv(self._val & other)
+
     def __rand__(self, other):
-        return type(self)(other & self._val)
+        return intbv(other & self._val)
 
     def __or__(self, other):
         if isinstance(other, intbv):
-            return type(self)(self._val | other._val)
+            return intbv(self._val | other._val)
         else:
-            return type(self)(self._val | other)
+            return intbv(self._val | other)
+
     def __ror__(self, other):
-        return type(self)(other | self._val)
-    
+        return intbv(other | self._val)
+
     def __xor__(self, other):
         if isinstance(other, intbv):
-            return type(self)(self._val ^ other._val)
+            return intbv(self._val ^ other._val)
         else:
-            return type(self)(self._val ^ other)
+            return intbv(self._val ^ other)
+
     def __rxor__(self, other):
-        return type(self)(other ^ self._val)
+        return intbv(other ^ self._val)
 
     def __iadd__(self, other):
         if isinstance(other, intbv):
@@ -299,7 +304,7 @@ class intbv(object):
             self._val += other
         self._handleBounds()
         return self
-        
+
     def __isub__(self, other):
         if isinstance(other, intbv):
             self._val -= other._val
@@ -307,7 +312,7 @@ class intbv(object):
             self._val -= other
         self._handleBounds()
         return self
-        
+
     def __imul__(self, other):
         if isinstance(other, intbv):
             self._val *= other._val
@@ -315,7 +320,7 @@ class intbv(object):
             self._val *= other
         self._handleBounds()
         return self
-    
+
     def __ifloordiv__(self, other):
         if isinstance(other, intbv):
             self._val //= other._val
@@ -326,9 +331,10 @@ class intbv(object):
 
     def __idiv__(self, other):
         raise TypeError("intbv: Augmented classic division not supported")
+
     def __itruediv__(self, other):
         raise TypeError("intbv: Augmented true division not supported")
-    
+
     def __imod__(self, other):
         if isinstance(other, intbv):
             self._val %= other._val
@@ -336,10 +342,10 @@ class intbv(object):
             self._val %= other
         self._handleBounds()
         return self
-        
+
     def __ipow__(self, other, modulo=None):
         # XXX why 3rd param required?
-        # unused but needed in 2.2, not in 2.3 
+        # unused but needed in 2.2, not in 2.3
         if isinstance(other, intbv):
             self._val **= other._val
         else:
@@ -348,7 +354,7 @@ class intbv(object):
             raise ValueError("intbv value should be integer")
         self._handleBounds()
         return self
-        
+
     def __iand__(self, other):
         if isinstance(other, intbv):
             self._val &= other._val
@@ -401,13 +407,13 @@ class intbv(object):
 
     def __invert__(self):
         if self._nrbits and self._min >= 0:
-            return type(self)(~self._val & (long(1) << self._nrbits)-1)
+            return intbv(~self._val & (long(1) << self._nrbits) - 1)
         else:
-            return type(self)(~self._val)
-    
+            return intbv(~self._val)
+
     def __int__(self):
         return int(self._val)
-        
+
     def __long__(self):
         return long(self._val)
 
@@ -415,109 +421,146 @@ class intbv(object):
         return float(self._val)
 
     # XXX __complex__ seems redundant ??? (complex() works as such?)
-    
+
     def __oct__(self):
         return oct(self._val)
-    
+
     def __hex__(self):
         return hex(self._val)
 
     def __index__(self):
         return int(self._val)
-        
+
     # comparisons
     def __eq__(self, other):
         if isinstance(other, intbv):
             return self._val == other._val
         else:
             return self._val == other
+
     def __ne__(self, other):
         if isinstance(other, intbv):
             return self._val != other._val
         else:
             return self._val != other
+
     def __lt__(self, other):
         if isinstance(other, intbv):
             return self._val < other._val
         else:
             return self._val < other
+
     def __le__(self, other):
         if isinstance(other, intbv):
             return self._val <= other._val
         else:
             return self._val <= other
+
     def __gt__(self, other):
         if isinstance(other, intbv):
             return self._val > other._val
         else:
             return self._val > other
+
     def __ge__(self, other):
         if isinstance(other, intbv):
             return self._val >= other._val
         else:
             return self._val >= other
 
-    # representation 
+    # representation
     def __str__(self):
-        return str(self._val)
+        # represent in hex format, to handle VHDL long vectors
+        v = int(self._val)
+        nrbits = self._nrbits
+        if nrbits:
+            # represent negative values without minus sign
+            # I would prefer sign extension with hex format, but to
+            # align with Verilog $display I don't do that
+            if v < 0:
+                v = 2**nrbits + v
+            w = (nrbits - 1) // 4 + 1
+            return "{:0{w}x}".format(v, w=w)
+        else:
+            return "{:x}".format(v)
 
     def __repr__(self):
         return "intbv(" + repr(self._val) + ")"
 
-
     def signed(self):
-      ''' return integer with the signed value of the intbv instance
+        ''' Return new intbv with the values interpreted as signed
 
-      The intbv.signed() function will classify the value of the intbv
-      instance either as signed or unsigned. If the value is classified
-      as signed it will be returned unchanged as integer value. If the
-      value is considered unsigned, the bits as specified by _nrbits
-      will be considered as 2's complement number and returned. This
-      feature will allow to create slices and have the sliced bits be
-      considered a 2's complement number.
+        The intbv.signed() function will classify the value of the intbv
+        instance either as signed or unsigned. If the value is classified
+        as signed it will be returned unchanged as integer value. If the
+        value is considered unsigned, the bits as specified by _nrbits
+        will be considered as 2's complement number and returned. This
+        feature will allow to create slices and have the sliced bits be
+        considered a 2's complement number.
+
+        The classification is based on the following possible combinations
+        of the min and max value.
 
-      The classification is based on the following possible combinations
-      of the min and max value.
-          
         ----+----+----+----+----+----+----+----
            -3   -2   -1    0    1    2    3
-      1                   min  max
-      2                        min  max
-      3              min       max
-      4              min            max
-      5         min            max
-      6         min       max
-      7         min  max
-      8   neither min nor max is set
-      9   only max is set
-      10  only min is set
+        1                   min  max
+        2                        min  max
+        3              min       max
+        4              min            max
+        5         min            max
+        6         min       max
+        7         min  max
+        8   neither min nor max is set
+        9   only max is set
+        10  only min is set
 
-      From the above cases, # 1 and 2 are considered unsigned and the
-      signed() function will convert the value to a signed number.
-      Decision about the sign will be done based on the msb. The msb is
-      based on the _nrbits value.
-      
-      So the test will be if min >= 0 and _nrbits > 0. Then the instance
-      is considered unsigned and the value is returned as 2's complement
-      number.
-      '''
+        From the above cases, # 1 and 2 are considered unsigned and the
+        signed() function will convert the value to a signed number.
+        Decision about the sign will be done based on the msb. The msb is
+        based on the _nrbits value.
 
-      # value is considered unsigned
-      if self.min is not None and self.min >= 0 and self._nrbits > 0:
+        So the test will be if min >= 0 and _nrbits > 0. Then the instance
+        is considered unsigned and the value is returned as 2's complement
+        number.
+        '''
 
-        # get 2's complement value of bits
-        msb = self._nrbits-1
+        # value is considered unsigned
+        if self.min is not None and self.min >= 0 and self._nrbits:
 
-        sign = ((self._val >> msb) & 0x1) > 0
-        
-        # mask off the bits msb-1:lsb, they are always positive
-        mask = (1<<msb) - 1
-        retVal = self._val & mask
-        # if sign bit is set, subtract the value of the sign bit
-        if sign:
-          retVal -= 1<<msb
+            # get 2's complement value of bits
+            msb = self._nrbits - 1
 
-      else: # value is returned just as is
-        retVal = self._val
+            sign = ((self._val >> msb) & 0x1) > 0
 
-      return retVal
+            # mask off the bits msb-1:lsb, they are always positive
+            mask = (1 << msb) - 1
+            retVal = self._val & mask
+            # if sign bit is set, subtract the value of the sign bit
+            if sign:
+                retVal -= 1 << msb
+
+        else:  # value is returned just as is
+            retVal = self._val
+
+        if self._nrbits:
+            M = 2**(self._nrbits - 1)
+            return intbv(retVal, min=-M, max=M)
+        else:
+            return intbv(retVal)
+
+    def unsigned(self):
+        ''' Return new intbv with the values interpreted as unsigned
+        '''
+
+        # value is considered unsigned
+        if self.min is not None and self.min < 0 and self._nrbits:
+            mask = (1 << self._nrbits) - 1
+            retVal = self._val & mask
+
+        else:  # value is returned just as is
+            retVal = self._val
+
+        if self._nrbits:
+            return intbv(retVal)[self._nrbits:]
+        else:
+            return intbv(retVal)        
