@@ -1,3 +1,4 @@
+import myhdl
 from myhdl import *
 
 from bin2gray2 import bin2gray
@@ -31,7 +32,7 @@ def main():
     graycnt = Signal(modbv(0)[width:])
     enable = Signal(bool())
     clock = Signal(bool())
-    reset = ResetSignal(0, active=0, async=True)
+    reset = ResetSignal(0, active=0, isasync=True)
 
     toVerilog(GrayIncReg, graycnt, enable, clock, reset, width)
     toVHDL(GrayIncReg, graycnt, enable, clock, reset, width)
